<?php

namespace Insign\PostCore\Controller\Adminhtml\Barcode;

use Exception;
use Insign\PostCore\Helper\BarcodeHelper;
use Insign\PostCore\Logger\Log;
use Insign\PostCore\Model\Config\Source\Barcode\LogoImage;
use Insign\PostCore\Util\OAuth2\AbstractProvider;
use Insign\PostCore\Util\OAuth2\AccessTokenManager;
use Insign\PostCore\Util\OAuth2\BarcodeProvider;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Filesystem\Io\File;
use Magento\Sales\Api\ShipmentRepositoryInterface;
use Magento\Sales\Model\OrderRepository;

class Save extends Action
{
    /** @var int */
    private $requestCounter = 0;

    /** @var BarcodeHelper  */
    protected $barcodeHelper;

    /** @var Log  */
    protected $logger;

    /** @var DirectoryList  */
    protected $directoryList;

    /** @var ShipmentRepositoryInterface  */
    protected $shipmentRepository;

    /** @var OrderRepository  */
    protected $orderRepository;

    /** @var Context */
    protected $context;

    /** @var  AccessTokenManager*/
    protected $accessTokenManager;

    /** @var BarcodeProvider  */
    protected $oauth2Provider;

    /** @var File */
    protected $file;

    /**
     * Save constructor.
     * @param Context $context
     * @param AccessTokenManager $accessTokenManager
     * @param BarcodeProvider $oauth2Provider
     * @param BarcodeHelper $barcodeHelper
     * @param OrderRepository $orderRepository
     * @param Log $logger
     * @param DirectoryList $directoryList
     * @param File $file
     * @param ShipmentRepositoryInterface $shipmentRepository
     */
    public function __construct(
        Context $context,
        AccessTokenManager $accessTokenManager,
        BarcodeProvider $oauth2Provider,
        BarcodeHelper $barcodeHelper,
        OrderRepository $orderRepository,
        Log $logger,
        DirectoryList $directoryList,
        File $file,
        ShipmentRepositoryInterface $shipmentRepository
    ) {
        $this->accessTokenManager = $accessTokenManager;
        $this->oauth2Provider = $oauth2Provider;
        $this->barcodeHelper = $barcodeHelper;
        $this->logger = $logger;
        $this->directoryList = $directoryList;
        $this->shipmentRepository = $shipmentRepository;
        $this->orderRepository = $orderRepository;
        parent::__construct($context);
    }

    /**
     * @inheritDoc
     * @throws Exception
     */
    public function execute()
    {
        $data = $this->getRequest()->getParams();
        $shipmentId = $data['shipment_id'];
        $shipment = $this->shipmentRepository->get($shipmentId);
        $orderId = $shipment->getOrderId();
        $this->_redirect('sales/shipment/view/shipment_id/' . $shipmentId);
        try {
            $order = $this->orderRepository->get($orderId);
        } catch (Exception $e) {
            $this->logger->error("Cant load order with order id: " . $orderId . " with the error message" . $e->getMessage());
            throw new Exception("Cant load order with order id: " . $orderId   . " with the error message: " . $e->getMessage());
        }

        $senderName = $this->barcodeHelper->getSenderName();
        $senderAddress = $this->barcodeHelper->getSenderAddress();
        $senderZip = $this->barcodeHelper->getSenderZip();
        $senderCity = $this->barcodeHelper->getSenderCity();

        if($data['sender_enable']){
            $senderName = $data["sender_name"];
            $senderAddress = $data['street'] . " " . $data["house_number"];
            $senderZip = $data["postcode"];
            $senderCity = $data["city"];
        }

        // Get the street number from the street field
        $recipientStreetNumber = '';
        $recipientStreet = [];
        $address = [];
        $address = $order->getShippingAddress();
        $recipientStreet = $address->getStreet();
        if (preg_match('/^([^\d]*[^\d\s]) *(\d.*)$/', reset($recipientStreet), $match)) {
            if (is_array($match)) {
                $recipientStreetNumber = end($match);
                $arrayStreet = array_slice($match, 1, 1, true);
                $recipientStreet = reset($arrayStreet);
            }
        }
        if(empty($recipientStreetNumber)){
            $this->logger->error("No valid street or street number is set");
            throw new Exception("No valid street or street number is set");
        }

        // Get Service Code and combine with Additional Services
        if(empty($data["service_code"])){
            $this->logger->error("Service Code was empty when creating an additional barcode");
            throw new Exception("Service Code was empty when creating an additional barcode");
        }


        $przl = $this->barcodeHelper->getServiceCode($order->getShippingMethod());
        $additionalCodes = $this->barcodeHelper->getAdditionalService();
        if(!is_array($data['service_code'])){
            $przl = explode(",", str_replace(' ', '', $data['service_code']));
        }
        if(!is_array($data['service_code'])) {
            $additionalCodes = $data['additional_service'];
        }

        foreach($additionalCodes as $code){
            if(!empty($code)){
                array_push($przl, $code);
            }
        }

        if(sizeof($przl) == 0){
            $this->logger->error("Service Code is not correctly set");
            throw new Exception("Service Code is not set.");
        }

        $barcodePayload = [
            'language' => 'DE',
            'frankingLicense' => $data['franking_license'],
            'customer' => [
                'name1' => $senderName,
                'street' => $senderAddress,
                'zip' => $senderZip,
                'city' => $senderCity,
                'domicilePostOffice' => $this->barcodeHelper->getDomicilePostOffice()
                // "country" => "CH",
            ],
            'labelDefinition' => [
                'labelLayout' => $this->barcodeHelper->getLabelLayout(),
                'printAddresses' => $this->barcodeHelper->getPrintAddress(),
                'imageFileType' => $this->barcodeHelper->getImageFiletype(),
                'imageResolution' => intval($this->barcodeHelper->getImageResolution())
            ],
            'item' => [
                'itemID' => $shipment->getIncrementId(),
                'recipient' => [
                    'name1' => $address->getFirstName() . ' ' . $address->getLastName(),
                    'street' => $recipientStreet,
                    'houseNo' => $recipientStreetNumber,
                    'zip' => $address->getPostcode(),
                    'city' => $address->getCity(),
                    'country' => $address->getCountryId(),
                ],
                'attributes' => [
                    'przl' => $przl,
                ],
            ]
        ];

        /** Add values if sender logo is set */
        try {
            $image = $this->barcodeHelper->getLogoImage();
            if ($image != null) {
                $filepath = $this->directoryList->getPath(DirectoryList::MEDIA) . "/" . LogoImage::UPLOAD_DIR . "/" .
                    $this->barcodeHelper->getLogoImage();
                if (file_exists($filepath)) {
                    $img = file_get_contents($filepath);
                    $imgData = base64_encode($img);
                    $senders = [
                        "logo" => $imgData,
                        "logoFormat" => strtoupper(pathinfo($this->barcodeHelper->getLogoImage(), PATHINFO_EXTENSION)),
                        "logoRotation" => $this->barcodeHelper->getLogoRotation(),
                        "logoAspectRatio" => $this->barcodeHelper->getLogoAspectRatio(),
                        "logoHorizontalAlign" => $this->barcodeHelper->getLogoHorizontalAlign(),
                        "logoVerticalAlign" => $this->barcodeHelper->getVerticalAlign()
                    ];
                    $barcodePayload['customer'] += $senders;
                }
            }
        } catch (Exception $e) {
            $this->logger->error("Barcode error when adding the Logo: " . $e->getMessage());
            throw new Exception("Barcode error when adding the Logo: " . $e->getMessage());
        }

        /** Add company if used */
        $company = $address->getCompany();
        if (!empty($company)) {
            $firstname = $barcodePayload['item']['recipient']['name1'];
            $barcodePayload['item']['recipient']['name1'] = $company;
            $barcodePayload['item']['recipient']['name2'] = $firstname;
        }

        /**
         * Get response from Swiss Post API
         */
        $url = $this->barcodeHelper->getEndpoint();
        $options['json'] = $barcodePayload;
        $accessToken = $this->accessTokenManager->getAccessToken($this->barcodeHelper->getScope());
        if (!isset($accessToken)) {
            $this->oauth2Provider->createAccessToken();
            $accessToken = $this->accessTokenManager->getAccessToken($this->barcodeHelper->getScope());
        }
        try {
            $response = $this->oauth2Provider->createAuthenticatedRequest(AbstractProvider::METHOD_HTTP_POST, $url, $accessToken, $options);

            if($response->getStatusCode() == 401 && $this->requestCounter < 1){
                $this->requestCounter++;
                $this->oauth2Provider->createAccessToken();
                return $this->execute($observer);
            }
            elseif ($response->getStatusCode() == 401 && $this->requestCounter == 1){
                $this->logger->error("Post API Response: " . $response->getStatusCode());
                throw new Exception("Post API Response: " . $response->getStatusCode());
            }

            $data = json_decode((string)$response->getBody(), true);

            if (isset($data['item']['label'][0])) {
                $filepath = $this->barcodeHelper->saveBarcodeToMedia($data['item']['label'][0], $shipment->getId());
                $this->barcodeHelper->SaveBarcode($filepath, $shipment->getId(), $shipment->getOrderId(), json_encode($barcodePayload), json_encode($data));
            }
            else if (isset($data['item']['errors'][0]['message'])){
                $this->logger->error("Wrong parameters: " . $data['item']['errors'][0]['message']);
                throw new Exception ($data['item']['errors'][0]['message']);
            }
            else{
                $this->logger->error("Error while creating barcode: " . $data['item']['errors']);
                throw new Exception ("Error while creating barcode" . $data['item']['errors']);
            }
        } catch (Exception $e) {
            if ($e->getCode() == 401 && $this->requestCounter < 1) {
                $this->requestCounter++;
                $this->oauth2Provider->createAccessToken();
                return $this->execute();
            } else {
                $this->logger->error('Unable to create barcode: ' . $e->getMessage());
                throw new Exception('Unable to create barcode: ' . $e->getMessage());
            }
        }
    }
}
