<?php

namespace Insign\PostCore\Helper;

use Exception;
use Insign\PostCore\Api\BarcodeRepositoryInterface;
use Insign\PostCore\Logger\Log;
use Insign\PostCore\Model\BarcodeFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Filesystem\Io\File;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Store\Model\ScopeInterface;

class BarcodeHelper
{
    const BARCODE_DIR = 'swisspost/barcode/codes';

    const XML_SWISS_POST_ADDRESS_BARCODE_ENABLE = 'swisspost/barcode_generation/barcode_enable';
    const XML_SWISS_POST_ADDRESS_BARCODE_SCOPE = 'swisspost/barcode_generation/scope';
    const XML_SWISS_POST_ADDRESS_BARCODE_ENDPOINT = 'swisspost/barcode_generation/endpoint';
    const XML_SWISS_POST_ADDRESS_BARCODE_FRANKING_LICENSE = 'swisspost/barcode_generation/franking_license';
    const XML_SWISS_POST_ADDRESS_BARCODE_FRANKING_LICENSE_LIST = 'swisspost/barcode_generation/franking_license_list';

    const XML_SWISS_POST_ADDRESS_BARCODE_SERVICE_CODE = 'swisspost/barcode_generation/service_code';
    const XML_SWISS_POST_ADDRESS_BARCODE_ADDITIONAL_SERVICE = 'swisspost/barcode_generation/additional_service';

    const XML_SWISS_POST_ADDRESS_BARCODE_ENABLE_CUSTOM_SENDER = 'swisspost/barcode_default_sender_settings/enable_custom_sender';

    const XML_SWISS_POST_ADDRESS_BARCODE_SENDER_NAME = 'swisspost/barcode_default_sender_settings/sender_name';
    const XML_SWISS_POST_ADDRESS_BARCODE_SENDER_ADDRESS = 'swisspost/barcode_default_sender_settings/sender_address';
    const XML_SWISS_POST_ADDRESS_BARCODE_SENDER_ZIP = 'swisspost/barcode_default_sender_settings/sender_zip';
    const XML_SWISS_POST_ADDRESS_BARCODE_SENDER_CITY = 'swisspost/barcode_default_sender_settings/sender_city';

    const XML_SWISS_POST_ADDRESS_BARCODE_LABEL_LAYOUT = 'swisspost/barcode_default_label_settings/label_layout';
    const XML_SWISS_POST_ADDRESS_BARCODE_PRINT_ADDRESSES = 'swisspost/barcode_default_label_settings/print_addresses';
    const XML_SWISS_POST_ADDRESS_BARCODE_IMAGE_FILETYPE = 'swisspost/barcode_default_label_settings/image_filetype';
    const XML_SWISS_POST_ADDRESS_BARCODE_IMAGE_RESOLUTION = 'swisspost/barcode_default_label_settings/image_resolution';

    const XML_SWISS_POST_ADDRESS_BARCODE_LOGO_IMAGE = 'swisspost/barcode_default_logo_settings/logo';
    const XML_SWISS_POST_ADDRESS_BARCODE_LOGO_ROTATION = 'swisspost/barcode_default_logo_settings/rotation';
    const XML_SWISS_POST_ADDRESS_BARCODE_LOGO_ASPECT_RATIO = 'swisspost/barcode_default_logo_settings/aspect_ratio';
    const XML_SWISS_POST_ADDRESS_BARCODE_LOGO_HORIZONTAL_ALIGN = 'swisspost/barcode_default_logo_settings/horizontal_align';
    const XML_SWISS_POST_ADDRESS_BARCODE_LOGO_VERTICAL_ALIGN = 'swisspost/barcode_default_logo_settings/vertical_align';

    /** @var ScopeConfigInterface  */
    protected $scopeConfig;

    /** @var Log  */
    protected $logger;

    /** @var DirectoryList */
    protected $directoryList;

    /** @var BarcodeRepositoryInterface */
    protected $barcodeRepository;

    /** @var BarcodeFactory */
    protected $barcodeFactory;

    /** @var File  */
    protected $file;
    /**
     * @var Json
     */
    private $serializer;

    /**
     * Barcode constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param DirectoryList $directoryList
     * @param BarcodeRepositoryInterface $barcodeRepository
     * @param BarcodeFactory $barcodeFactory
     * @param File $file
     * @param Log $logger
     * @param Json $serializer
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        DirectoryList $directoryList,
        BarcodeRepositoryInterface $barcodeRepository,
        BarcodeFactory $barcodeFactory,
        File $file,
        Log $logger,
        Json $serializer
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->directoryList = $directoryList;
        $this->barcodeRepository = $barcodeRepository;
        $this->barcodeFactory = $barcodeFactory;
        $this->file = $file;
        $this->logger = $logger;
        $this->serializer = $serializer;
    }

    /**
     * @return bool
     */
    public function isEnable(): bool
    {
        $status =  $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_ENABLE, ScopeInterface::SCOPE_WEBSITE);
        if ($status > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @return mixed
     */
    public function getScope()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_SCOPE, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getEndpoint()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_ENDPOINT, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return array
     * @throws Exception
     */
    public function getFrankingLicenseList()
    {
        $jsonString = $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_FRANKING_LICENSE_LIST, ScopeInterface::SCOPE_WEBSITE);
        try {
            if (is_string($jsonString) && !empty($jsonString)) {
                return $this->serializer->unserialize($jsonString);
            }
        } catch (Exception $e) {
            throw new Exception('something is wrong with franking license list ' . $e->getMessage());
        }
    }

    /**
     * @return mixed
     */
    public function getFrankingLicense()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_FRANKING_LICENSE, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @param string $shippingMethod
     * @return array
     */
    public function getServiceCode(string $shippingMethod): array
    {
        $serviceCodes = $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_SERVICE_CODE, ScopeInterface::SCOPE_WEBSITE);
        $serviceCodes = json_decode($serviceCodes);

        foreach ($serviceCodes as $code) {
            if (get_object_vars($code)['shipping_methods'] == $shippingMethod) {
                $codes = get_object_vars($code)['service_code'];
                return explode(",", str_replace(' ', '', $codes));
            }
        }
        return [];
    }

    /**
     * @return mixed
     */
    public function getAdditionalService()
    {
        $codes = $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_ADDITIONAL_SERVICE, ScopeInterface::SCOPE_WEBSITE);
        return explode(",", str_replace(' ', '', $codes));
    }

    /**
     * @return mixed
     */
    public function isCustomSenderEnabled()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_ENABLE_CUSTOM_SENDER, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getSenderName()
    {
        if ($this->isCustomSenderEnabled()) {
            return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_SENDER_NAME, ScopeInterface::SCOPE_WEBSITE);
        } else {
            return $this->scopeConfig->getValue("general/store_information/name", ScopeInterface::SCOPE_WEBSITE);
        }
    }

    /**
     * @return mixed
     */
    public function getSenderAddress()
    {
        if ($this->isCustomSenderEnabled()) {
            return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_SENDER_ADDRESS, ScopeInterface::SCOPE_WEBSITE);
        } else {
            return $this->scopeConfig->getValue("general/store_information/street_line1", ScopeInterface::SCOPE_WEBSITE);
        }
    }

    /**
     * @return mixed
     */
    public function getSenderZip()
    {
        if ($this->isCustomSenderEnabled()) {
            return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_SENDER_ZIP, ScopeInterface::SCOPE_WEBSITE);
        } else {
            return $this->scopeConfig->getValue("general/store_information/postcode", ScopeInterface::SCOPE_WEBSITE);
        }
    }

    /**
     * @return mixed
     */
    public function getSenderCity()
    {
        if ($this->isCustomSenderEnabled()) {
            return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_SENDER_CITY, ScopeInterface::SCOPE_WEBSITE);
        } else {
            return $this->scopeConfig->getValue("general/store_information/city", ScopeInterface::SCOPE_WEBSITE);
        }
    }

    /**
     * @return string
     */
    public function getDomicilePostOffice(): string
    {
        if ($this->isCustomSenderEnabled()) {
            return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_SENDER_ZIP, ScopeInterface::SCOPE_WEBSITE) . " " . $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_SENDER_CITY, ScopeInterface::SCOPE_WEBSITE);
        } else {
            return $this->scopeConfig->getValue("general/store_information/postcode", ScopeInterface::SCOPE_WEBSITE) . " " . $this->scopeConfig->getValue("general/store_information/city", ScopeInterface::SCOPE_WEBSITE);
        }
    }

    /**
     * @return mixed
     */
    public function getLabelLayout()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_LABEL_LAYOUT, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getPrintAddress()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_PRINT_ADDRESSES, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getImageFiletype()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_IMAGE_FILETYPE, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getImageResolution()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_IMAGE_RESOLUTION, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getLogoImage()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_LOGO_IMAGE, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getLogoRotation()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_LOGO_ROTATION, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getLogoAspectRatio()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_LOGO_ASPECT_RATIO, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getLogoHorizontalAlign()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_LOGO_HORIZONTAL_ALIGN, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @return mixed
     */
    public function getVerticalAlign()
    {
        return $this->scopeConfig->getValue($this::XML_SWISS_POST_ADDRESS_BARCODE_LOGO_VERTICAL_ALIGN, ScopeInterface::SCOPE_WEBSITE);
    }

    /**
     * @param string $filePath
     * @param int $shippingId
     * @param int $orderId
     * @param string $request
     * @param string $response
     */
    public function SaveBarcode(string $filePath, int $shippingId, int $orderId, string $request, string $response)
    {
        $barcode = $this->barcodeFactory->create();
        $barcode->setPath($filePath);
        $barcode->setShippingId($shippingId);
        $barcode->setOrderId($orderId);
        $barcode->setRequest($request);
        $barcode->setResponse($response);
        $barcode = $this->barcodeRepository->save($barcode);
    }

    /**
     * @param string $imageData
     * @param int $shipmentId
     * @return string
     * @throws Exception
     */
    public function saveBarcodeToMedia(string $imageData, int $shipmentId): string
    {
        try {
            if (! file_exists($this->directoryList->getPath(DirectoryList::MEDIA) . "/" . $this::BARCODE_DIR)) {
                $this->file->mkdir($this->directoryList->getPath(DirectoryList::MEDIA) . "/" . $this::BARCODE_DIR, 0775);
            }
            $path = $this->directoryList->getPath(DirectoryList::MEDIA) . "/" . $this::BARCODE_DIR . "/";
            $filename = $shipmentId . time() . "." . $this->getImageFiletype();
            file_put_contents($path . $filename, base64_decode($imageData));
            return "/" . $this::BARCODE_DIR . "/" . $filename;
        } catch (Exception $e) {
            $this->logger->error("Unable to save barcode: " . $e->getMessage());
            throw new Exception("Unable to save barcode: " . $e->getMessage());
        }
    }
}
