<?php

namespace Insign\PostCore\Observer;

use Exception;
use Insign\PostCore\Logger\Log;
use Insign\PostCore\Model\Barcode;
use Insign\PostCore\Model\BarcodeRepository;
use Insign\PostCore\Model\Config\Source\Barcode\LogoImage;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Event\Observer as EventObserver;
use Magento\Framework\Event\ObserverInterface;
use Insign\PostCore\Util\OAuth2\AbstractProvider;
use Insign\PostCore\Util\OAuth2\BarcodeProvider;
use Insign\PostCore\Util\OAuth2\AccessTokenManager;
use Insign\PostCore\Helper\BarcodeHelper;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Model\OrderRepository;
use Magento\Framework\Filesystem\Io\File;
use Psr\Http\Message\ResponseInterface;

class BarcodeGeneration implements ObserverInterface
{

    /** @var int */
    private $requestCounter = 0;

    /** @var BarcodeProvider  */
    protected $oauth2Provider;

    /** @var  AccessTokenManager*/
    protected $accessTokenManager;

    /** @var Log  */
    protected $logger;

    /** @var BarcodeHelper */
    protected $barcodeHelper;

    /** @var File  */
    protected $file;

    /** @var OrderRepository */
    protected $order;

    /** @var SearchCriteriaBuilder  */
    protected $searchCriteriaBuilder;

    /** @var FilterBuilder  */
    protected $filterBuilder;

    /** @var BarcodeRepository  */
    protected $barcodeRepository;

    protected $directoryList;

    /**
     * BarcodeGeneration constructor.
     * @param AccessTokenManager $accessTokenManager
     * @param BarcodeProvider $oauth2Provider
     * @param BarcodeHelper $barcodeHelper
     * @param BarcodeRepository $barcodeRepository
     * @param OrderRepository $order
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param FilterBuilder $filterBuilder
     * @param DirectoryList $directoryList
     * @param File $file
     * @param Log $logger
     */
    public function __construct(
        AccessTokenManager $accessTokenManager,
        BarcodeProvider $oauth2Provider,
        BarcodeHelper $barcodeHelper,
        BarcodeRepository $barcodeRepository,
        OrderRepository $order,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        FilterBuilder $filterBuilder,
        DirectoryList $directoryList,
        File $file,
        Log $logger
    ) {
        $this->accessTokenManager = $accessTokenManager;
        $this->oauth2Provider = $oauth2Provider;
        $this->barcodeHelper = $barcodeHelper;
        $this->barcodeRepository = $barcodeRepository;
        $this->order = $order;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterBuilder = $filterBuilder;
        $this->directoryList = $directoryList;
        $this->file = $file;
        $this->logger = $logger;
    }

    /**
     * @param EventObserver $observer
     * @return ResponseInterface|string|void
     * @throws Exception
     */
    public function execute(EventObserver $observer)
    {
        $shipment = $observer->getEvent()->getShipment();
        if (!($this->hasBarcodeGenerated($shipment->getId())) && ($this->barcodeHelper->isEnable())) {
            try {
                $order = $this->order->get($shipment->getOrderId());
            } catch (Exception $e) {
                $this->logger->error("Barcode Error when trying to load the order with the ID: " . $shipment->getOrderId() . " with the Error: " . $e->getMessage());
                throw new Exception("Can't load order with id " . $shipment->getOrderId()  . " error message: " . $e->getMessage());
            }

            // Get the street number from the street field
            $recipientStreetNumber = '';
            $recipientStreet = [];
            $address = [];
            $address = $order->getShippingAddress();
            $recipientStreet = $address->getStreet();
            if (preg_match('/^([^\d]*[^\d\s]) *(\d.*)$/', reset($recipientStreet), $match)) {
                if (is_array($match)) {
                    $recipientStreetNumber = end($match);
                    $arrayStreet = array_slice($match, 1, 1, true);
                    $recipientStreet = reset($arrayStreet);
                }
            }
            if(empty($recipientStreetNumber)){
                $this->logger->error("No valid street or street number is set");
                throw new Exception("No valid street or street number is set");
            }

            // Get Service Code and combine with Additional Services
            $przl = $this->barcodeHelper->getServiceCode($order->getShippingMethod());
            $additionalCodes = $this->barcodeHelper->getAdditionalService();

            foreach($additionalCodes as $code){
                if(!empty($code)){
                    array_push($przl, $code);
                }
            }

            if(sizeof($przl) == 0){
                $this->logger->error("Service Code is not correctly set");
                throw new Exception("Service Code is not set.");
            }

            $barcodePayload = [
                'language' => 'DE',
                'frankingLicense' => $this->barcodeHelper->getFrankingLicense(),
                'customer' => [
                    'name1' => $this->barcodeHelper->getSenderName(),
                    'street' => $this->barcodeHelper->getSenderAddress(),
                    'zip' => $this->barcodeHelper->getSenderZip(),
                    'city' => $this->barcodeHelper->getSenderCity(),
                    'domicilePostOffice' => $this->barcodeHelper->getDomicilePostOffice()
                    // "country" => "CH",
                ],
                'labelDefinition' => [
                    'labelLayout' => $this->barcodeHelper->getLabelLayout(),
                    'printAddresses' => $this->barcodeHelper->getPrintAddress(),
                    'imageFileType' => $this->barcodeHelper->getImageFiletype(),
                    'imageResolution' => intval($this->barcodeHelper->getImageResolution())
                ],
                'item' => [
                    'itemID' => $shipment->getIncrementId(),
                    'recipient' => [
                        'name1' => $address->getFirstName() . ' ' . $address->getLastName(),
                        'street' => $recipientStreet,
                        'houseNo' => $recipientStreetNumber,
                        'zip' => $address->getPostcode(),
                        'city' => $address->getCity(),
                        'country' => $address->getCountryId(),
                    ],
                    'attributes' => [
                        'przl' => $przl,
                    ],
                ]
            ];

            /** Add values if sender logo is set */
            try {
                $image = $this->barcodeHelper->getLogoImage();
                if ($image != null) {
                    $filepath = $this->directoryList->getPath(DirectoryList::MEDIA) . "/" . LogoImage::UPLOAD_DIR . "/" .
                        $this->barcodeHelper->getLogoImage();
                    if (file_exists($filepath)) {
                        $img = file_get_contents($filepath);
                        $data = base64_encode($img);
                        $senders = [
                            "logo" => $data,
                            "logoFormat" => strtoupper(pathinfo($this->barcodeHelper->getLogoImage(), PATHINFO_EXTENSION)),
                            "logoRotation" => $this->barcodeHelper->getLogoRotation(),
                            "logoAspectRatio" => $this->barcodeHelper->getLogoAspectRatio(),
                            "logoHorizontalAlign" => $this->barcodeHelper->getLogoHorizontalAlign(),
                            "logoVerticalAlign" => $this->barcodeHelper->getVerticalAlign()
                        ];
                        $barcodePayload['customer'] += $senders;
                    }
                }
            } catch (Exception $e) {
                $this->logger->error("Barcode error when adding the Logo" . $e->getMessage());
                throw new Exception("Barcode error when adding the Logo: " . $e->getMessage());
            }

            /** Add company if used */
            $company = $address->getCompany();
            if (!empty($company)) {
                $firstname = $barcodePayload['item']['recipient']['name1'];
                $barcodePayload['item']['recipient']['name1'] = $company;
                $barcodePayload['item']['recipient']['name2'] = $firstname;
            }

            /**
             * Get response from Swiss Post API
             */
            $url = $this->barcodeHelper->getEndpoint();
            $options['json'] = $barcodePayload;
            $accessToken = $this->accessTokenManager->getAccessToken($this->barcodeHelper->getScope());
            if (!isset($accessToken)) {
                $this->oauth2Provider->createAccessToken();
                $accessToken = $this->accessTokenManager->getAccessToken($this->barcodeHelper->getScope());
            }
            try {
                $response = $this->oauth2Provider->createAuthenticatedRequest(AbstractProvider::METHOD_HTTP_POST, $url, $accessToken, $options);

                if($response->getStatusCode() == 401 && $this->requestCounter < 1){
                    $this->requestCounter++;
                    $this->oauth2Provider->createAccessToken();
                    return $this->execute($observer);
                }
                elseif ($response->getStatusCode() == 401 && $this->requestCounter == 1){
                    $this->logger->error("Post API Response: " . $response->getStatusCode());
                    throw new Exception("Post API Response: " . $response->getStatusCode());
                }

                $data = json_decode((string)$response->getBody(), true);

                if (isset($data['item']['label'][0])) {
                    $filepath = $this->barcodeHelper->saveBarcodeToMedia($data['item']['label'][0], $shipment->getId());
                    $this->barcodeHelper->SaveBarcode($filepath, $shipment->getId(), $shipment->getOrderId(), json_encode($barcodePayload), json_encode($data));
                }
                else if (isset($data['item']['errors'][0]['message'])){
                    $this->logger->error("Wrong parameters: " . $data['item']['errors'][0]['message']);
                    throw new Exception ($data['item']['errors'][0]['message']);
                }
                else{
                    $this->logger->error("Error while creating barcode: " . $data['item']['errors']);
                    throw new Exception ("Error while creating barcode" . $data['item']['errors']);
                }
            } catch (Exception $e) {
                if ($e->getCode() == 401 && $this->requestCounter < 1) {
                    $this->requestCounter++;
                    $this->oauth2Provider->createAccessToken();
                    return $this->execute($observer);
                } else {
                    throw new Exception('Unable to create barcode. ' . $e->getMessage());
                }
            }
        }
    }

    public function hasBarcodeGenerated(int $shipmentId): bool
    {
        $filter = $this->filterBuilder
            ->setField(Barcode::SHIPPING_ID)
            ->setConditionType('eq')
            ->setValue($shipmentId)
            ->create();
        $this->searchCriteriaBuilder->addFilters([$filter]);
        $searchCriteria = $this->searchCriteriaBuilder->create();
        $barcodeItems  = $this->barcodeRepository->getList($searchCriteria)->getItems();

        if(sizeof($barcodeItems) > 0){
            return true;
        }
        else{
            return false;
        }
    }
}
