<?php

namespace Insign\PostCore\Setup;

use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\Setup\UpgradeSchemaInterface;
use Zend_Db_Exception;

class UpgradeSchema implements UpgradeSchemaInterface
{

    /**
     * Upgrades DB schema for a module
     *
     * @param SchemaSetupInterface $setup
     * @param ModuleContextInterface $context
     * @return void
     * @throws Zend_Db_Exception
     */
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();

        if (version_compare($context->getVersion(), '0.0.4', '<')) {
            $this->addBarcodeTable($setup);
            $this->addSwissConfigTable($setup);
            $this->addColumnAddressValidationStatusToQuoteAddress($setup);
            $this->addColumnAddressValidationStatusToSalesOrderAddress($setup);
        }

        $setup->endSetup();
    }

    /**
     * Creates a new table called `insign_swiss_post_barcode` which holds information about Barcode Generation Swiss Post API
     *
     * @param SchemaSetupInterface $setup
     * @throws Zend_Db_Exception
     */
    private function addBarcodeTable(SchemaSetupInterface $setup) : void
    {
        $setup->getConnection()->createTable($setup->getConnection()
            ->newTable($setup->getTable('insign_swiss_post_barcode'))
            ->addColumn(
                'entity_id',
                Table::TYPE_INTEGER,
                null,
                ['identity'  => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Entity Id'
            )->addColumn(
                'shipment_id',
                Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => false],
                'Magento Shipment Id'
            )->addColumn(
                'order_id',
                Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => false],
                'Magento Order Id'
            )->addColumn(
                'request',
                Table::TYPE_TEXT,
                '64k',
                [],
                'Barcode Request'
            )->addColumn(
                'response',
                Table::TYPE_TEXT,
                '64k',
                [],
                'Barcode Response'
            )->addColumn(
                'label_file_path',
                Table::TYPE_TEXT,
                255,
                ['nullable' => false],
                'Label File Path'
            )->addColumn(
                'created_at',
                Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => Table::TIMESTAMP_INIT],
                'Created At'
            )->addIndex(
                $setup->getIdxName('insign_swiss_post_barcode', ['created_at']),
                ['created_at']
            )->setComment('insign Swiss Post Barcode'));

    }

    /**
     * Creates a new table called `insign_swiss_config` which holds information about access tokens and scopes
     * @param SchemaSetupInterface $setup
     * @throws Zend_Db_Exception
     */
    private function addSwissConfigTable(SchemaSetupInterface $setup):void
    {
        $setup->getConnection()->createTable($setup->getConnection()
            ->newTable($setup->getTable('insign_swiss_config'))
            ->addColumn(
                'entity_id',
                Table::TYPE_INTEGER,
                null,
                ['identity'  => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Id'
            )->addColumn(
                'access_token',
                Table::TYPE_TEXT,
                '64k',
                ['nullable' => false],
                'Access Token'
            )->addColumn(
                'scope',
                Table::TYPE_TEXT,
                255,
                ['nullable' => false],
                'Scope'
            )->addColumn(
                'created_at',
                Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => Table::TIMESTAMP_INIT],
                'Created At'
            )->addColumn(
                'updated_at',
                Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => Table::TIMESTAMP_INIT],
                'Updated at At'
            )->setComment('insign Swiss Config'));
    }

    /**
     * Adding column address_validation_status to quote_address table
     * @param SchemaSetupInterface $setup
     */
    private function addColumnAddressValidationStatusToQuoteAddress(SchemaSetupInterface $setup):void
    {
        $setup->getConnection()->addColumn(
            $setup->getTable('quote_address'),
            'address_validation_status',
            [
                'type' => Table::TYPE_TEXT,
                'length' => 255,
                'nullable' => true,
                'comment' => 'Address Validation Status'
            ]
        );
    }

    /**
     * Adding column address_validation_status to sales_order_address table
     * @param SchemaSetupInterface $setup
     */
    private function addColumnAddressValidationStatusToSalesOrderAddress(SchemaSetupInterface $setup):void
    {
        $setup->getConnection()->addColumn(
            $setup->getTable('sales_order_address'),
            'address_validation_status',
            [
                'type' => Table::TYPE_TEXT,
                'length' => 255,
                'nullable' => true,
                'comment' => 'Address Validation Status'
            ]
        );
    }
}
